Attribute VB_Name = "Plotting"
Option Explicit

Global RefBp As Single

Type Values     'Returned by GetValVsF()
    'Coeffs to display full scale
    MuK As Single
    BK As Single
    LpK As Single
    ZpK As Single
    BpK As Single
    PhK As Single
    'Values themselves
    MuV As Single
    BV As Single
    LpV As Single
    ZpV As Single
    BpV As Single
    PhV As Single
    'Max for relative gain
    MaxGain As Single
    RefGain As Single
End Type

Type Complex
    re As Double
    im As Double
End Type

Dim ITLoad As Complex

Type XNet
    Net As Complex
    Load As Complex
End Type

Type Polar
    Z As Single
    T As Single
End Type

Type Link       'For each link between two nodes
    From As String
    To As String
    X As Complex
End Type

Dim Links() As Link

Const PtoPlot = 104

Dim Flist(PtoPlot) As Single    'The list of frequencies to use
Dim LogFlist(PtoPlot) As Single 'Precalculated log values for where to display
Global BMuColor As Long
Dim BMuK As Single
Global LpColor As Long
Global ZpColor As Long
Global PhColor As Long
Global BpColor As Long

Public Function RtoCpx(R As Single, F As Single) As Complex

    RtoCpx.re = R
    RtoCpx.im = 0
End Function

Public Function CtoCpx(C As Single, F As Single) As Complex

    CtoCpx.re = 0
    CtoCpx.im = 1 / C * 2 * pi * F
End Function
Public Function LtoCpx(L As Single, F As Single) As Complex

    LtoCpx.re = 0
    LtoCpx.im = L * 2 * pi * F
End Function

Public Function CpxAdd(A As Complex, b As Complex) As Complex

'Debug.Print "CPXAdd"

CpxAdd.re = A.re + b.re
CpxAdd.im = A.im + b.im

End Function

Public Function CpxMul(A As Complex, b As Complex) As Complex

CpxMul.re = A.re * b.re - A.im * b.im
CpxMul.im = A.re * b.im + A.im * b.re

End Function

Public Function Conjugate(A As Complex) As Complex

Conjugate.re = A.re
Conjugate.im = -A.im

End Function

Private Function CpxDiv(A As Complex, b As Complex) As Complex
Dim Num As Complex
Dim Den As Complex

    On Error GoTo DIVERROR
    Num = CpxMul(A, Conjugate(b))
    Den = CpxMul(b, Conjugate(b))
    CpxDiv.re = Num.re / Den.re
    CpxDiv.im = Num.im / Den.re
    Exit Function
DIVERROR:
    SetFatal "DIVERR"
    Resume Next
End Function

Private Function CpxPar(A As Complex, b As Complex) As Complex

    If A.re = 0 And A.im = 0 Then
        CpxPar = b
        Exit Function
    End If
    If b.re = 0 And b.im = 0 Then
        CpxPar = A
        Exit Function
    End If
    CpxPar = CpxDiv(CpxMul(A, b), CpxAdd(A, b))
End Function

Private Function CpxRatio(A As Complex, b As Complex) As Complex

    CpxRatio = CpxDiv(b, CpxAdd(A, b))
End Function
Public Function Modulo(A As Complex) As Double

    On Error GoTo MODULOERR
    Modulo = Sqr(A.re ^ 2 + A.im ^ 2)   'Pythagore !
    Exit Function
MODULOERR:
    SetFatal "MODULOERR"
    Resume Next
End Function

Private Function Argument(A As Complex) As Double

    If A.re <> 0 Then
        Argument = (180 / pi) * Atn(A.im / A.re) 'OK even if im is 0
        If A.re < 0 Then Argument = 180 + Argument
    Else    'when real part is nul
        If A.im > 0 Then Argument = 90
        If A.im < 0 Then Argument = -90
    End If
End Function

Public Function CtoP(A As Complex) As Polar
' Convert complex to polar
    CtoP.Z = Modulo(A)
    CtoP.T = Argument(A)
End Function

Public Function PtoC(A As Polar) As Complex
'TODO: Convert polar to complex


End Function

'Public Sub SetPlotTarget(Target As PictureBox)
'    PlotTo = Target
'End Sub
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Public Sub ShowFVPlot(X As Single, Y As Single, FL As Label, VL As Label)
'    Form1.Label41.Caption = Format(10 * (2.3025851 ^ (1.2 * X)), "#.##")   'X
    If X >= 0 Then FL.Caption = UFormat(10 * (2.3025851 ^ (1.2 * X)))
    If Y >= 0 Then VL.Caption = Format(Y / 10, "0.0#")
End Sub

Public Sub InitPlot(Target As PictureBox)
'Draw graticle in the plot picture box
Dim I As Integer
Dim J As Single
Dim X As Single
Dim Y As Single
Dim T As String
Dim GC As Long      'Grid Color
Dim F As Single

    Target.Cls
    Target.DrawMode = 13    'copy pen
    Target.Scale (0, 10.5)-(10, -0.5)
    Target.BackColor = RGB(255, 255, 255)
    Target.ForeColor = RGB(0, 0, 0)
    GC = RGB(192, 192, 192)   'Light grey
        
    ' Draw X axis. One loop for each of 4 decades
    Target.CurrentX = 0
    Target.CurrentY = 0
    
    For I = 2 To 10
        J = I
        X = Log(I)
        Target.Line (X, 10)-(X, 0), GC      'RGB(128, 128, 128)
    Next I
    Target.CurrentY = 0 'Legend at bottom
    T = UFormat(J * 10)
    Target.CurrentX = Target.CurrentX - Target.TextWidth(T) / 2 'Center legend
    Target.Print T
    
    For I = 20 To 100 Step 10
        J = I
        X = Log(I)
        Target.Line (X, 10)-(X, 0), GC      'RGB(128, 128, 128)
    Next I
    Target.CurrentY = 0
    T = UFormat(J * 10)
    Target.CurrentX = Target.CurrentX - Target.TextWidth(T) / 2
    Target.Print T
    
    For I = 200 To 1000 Step 100
        J = I
        X = Log(I)
        Target.Line (X, 10)-(X, 0), GC      'RGB(128, 128, 128)
    Next I
    Target.CurrentY = 0
    T = UFormat(J * 10)
    Target.CurrentX = Target.CurrentX - Target.TextWidth(T) / 2
    Target.Print T

    For I = 2000 To 10000 Step 1000
        J = I
        X = Log(I)
        Target.Line (X, 10)-(X, 0), GC      'RGB(128, 128, 128)
    Next I
    Target.CurrentY = 0
    T = UFormat(J * 10)
    Target.CurrentX = Target.CurrentX - Target.TextWidth(T) / 2
    Target.Print T

    Target.Line (0, 0)-(10, 0), GC      'RGB(128, 128, 128)
    
    ' Draw dB Y axis.
    Target.Line (0, 0)-(0, 10), GC
    For I = 0 To 10
        Target.Line (10, I)-(0, I), GC
    Next I

''' Build the lin and log frequency steps lists
    F = 10
    For I = 0 To PtoPlot
        Flist(I) = F
        LogFlist(I) = Log(F / 10)
        F = F * 1.1
    Next
    Target.ForeColor = RGB(0, 0, 0)
    Target.DrawMode = 13     'Redraw the cursor
    Target.Line (PlotCurrentX, 0)-(PlotCurrentX, 10)
End Sub

Function Log10(V As Double) As Double
    If V = 0 Then Exit Function
    Log10 = 0.434294481903252 * Log(V)
End Function

Function UFormat(U As Single) As String
Dim Unit As String
Dim X As Single

    Select Case U
    Case Is >= 1000000
        X = U / 1000000
        Unit = "M"
    Case Is >= 1000
        X = U / 1000
        Unit = "K"
    Case Is <= 0.001
        X = 1000 * U
        Unit = "m"
    Case Else
        X = U
    End Select
    UFormat = Format(X, "#.#") & Unit
End Function

Public Sub DoPlotB(Target As PictureBox, Optional SimulModel As String)

Dim I As Integer
Dim F As Single
Dim Max As Single
Dim AC As Single
Dim DC As Single
Dim Vac As Single
Dim Mu As Single
Dim Gap As Single
Dim BValues(PtoPlot) As Single
Dim MaxB As Single
Dim MuValues(PtoPlot) As Single
Dim MaxMu As Single
Dim MaxBp As Single
Dim LpValues(PtoPlot) As Single
Dim MaxLp As Single
Dim PhValues(PtoPlot) As Single
Dim BpValues(PtoPlot) As Single
Dim ZpValues(PtoPlot) As Single
Dim MaxZp As Single
Dim R As Values 'results from call to GetValVsF()


    
    On Error GoTo DOPLOTBERR
'    PlotCurrentX = 0
''' Any way prepare new lists of values to be ploted according to frequency steps
    
    If Len(Form1.PlotVsource.Text) = 0 Then Form1.PlotVsource.Text = Format(TbV(0), "#.#")
    If Len(Form1.PlotIdc.Text) = 0 Then Form1.PlotIdc.Text = Form1.TxIp0.Text
    If Len(Form1.PlotZsrc.Text) = 0 Then Form1.PlotZsrc.Text = Form1.TxRp.Text & "R"
    If Len(Form1.PlotZld.Text) = 0 Then Form1.PlotZld.Text = Form1.TxZ(1).Text & "R"
    Vac = Val(Form1.PlotVsource.Text)
    Form1.PlotWatts.Caption = Format((Vac ^ 2) / Form1.TxZ(0).Text, "0.0# W")

    'UFormat((Vac ^ 2) / Form1.TxZ(0).Text) & "W"
    
    MaxB = 0
    MaxMu = 0
    MaxLp = 0
    R = GetValVsF(1000)   'Reference Bp at 1Khz = 0dB
    RefBp = R.BpV   ' - 5       'Midscale
    
    For I = 0 To PtoPlot    'Compute and store values for each frequency in the list
        If F >= 1000 Then
'            Beep            ' to test at 1000Hz
        End If
        F = Flist(I)
        R = GetValVsF(F)
        PhValues(I) = -R.PhV + 180   'Center 0
        BpValues(I) = R.BpV - RefBp + 5  'Center and scale for 1dB/Div
        ZpValues(I) = R.ZpV
        If ZpValues(I) > MaxZp Then MaxZp = ZpValues(I)
'        If R.BpV > MaxBp Then MaxBp = R.BpV
        BValues(I) = R.BV
        If R.BV > MaxB Then MaxB = R.BV
'        Mu = Extrapol(BValues(I))
'        Gap = Val(Form1.TxGap.Text / 10) 'Displayed in mm, convert in cm
'        Mu = Mu / (1 + (Mu * (Gap / FeLen)))
'        If Form1.ChkSE.Value Then
'            DC = Mu * Format(Form1.TxIp0.Text) * TbTurns(0) / FeLen / 10000      'Add DC flux
'            Mu = Extrapol(AC + DC)
'            Mu = Mu / (1 + (Mu * (Gap / FeLen)))
'        End If
''        MuValues(I) = Mu
        MuValues(I) = R.MuV
        If R.MuV > MaxMu Then MaxMu = R.MuV
''             LP = Perm * Spires ^ 2 * FeSect * 10 ^ -8 / FeLen
'        LpValues(I) = 100 + Mu * TbTurns(0) ^ 2 * FeSect * 10 ^ -8 / FeLen
        LpValues(I) = R.LpV
        If LpValues(I) > MaxLp Then MaxLp = LpValues(I)
'        SimValues(I) = SimV
    Next
    PlotToFile BValues, MuValues, ZpValues, LpValues, PhValues, BpValues
    R.BK = 1
    If MaxB <> 0 Then R.BK = 10 / MaxB      'Auto full scale
    R.MuK = 1
    If MaxMu <> 0 Then R.MuK = 10 / MaxMu
    R.LpK = 1
    If MaxLp <> 0 Then R.LpK = 10 / MaxLp
    R.ZpK = 1
    If MaxZp <> 0 Then R.ZpK = 10 / MaxZp
    If Form1.Label45.Tag <> 0 Then XorPlot Target, BValues, R.BK, BMuColor     'Go retrace new values
    If Form1.Label46.Tag <> 0 Then XorPlot Target, MuValues, R.MuK, BMuColor
    If Form1.Label48.Tag <> 0 Then XorPlot Target, ZpValues, R.ZpK, ZpColor
    If Form1.Label47.Tag <> 0 Then XorPlot Target, LpValues, R.LpK, LpColor
    If Form1.Label49.Tag <> 0 Then XorPlot Target, PhValues, 1 / 36, PhColor
    If Form1.Label50.Tag <> 0 Then XorPlot Target, BpValues, 1, BpColor
'    If Len(Fatal) Then  'Show error and abort
    ShowFatal
'        Exit Function
'    End If
    
    Exit Sub
DOPLOTBERR:
    SetFatal "DOPLOTBERR"
End Sub
Public Function GetValVsF(F As Single) As Values

Dim Source As Complex
Dim Neta As Complex
Dim Netb As Complex
Dim NetIn As Complex    'Including the source impedance
Dim Load As Complex
Dim Temp As Complex
'Dim X As XNet
Dim Dummy As Double
Dim Omega As Single 'Pulsation

Dim Mu As Single
Dim b As Single 'Induction
Dim bac As Single
Dim Gap As Single
Dim V As Single
Dim DC As Single
Dim AC As Single
Dim Rtot As String
Dim S As String
Dim P As Polar
Dim Lpri As String
Dim LLeak As String
Dim CShunt As String
Dim Leak As Complex
Dim CLump As String
Dim LLump As String
Dim Lump As Complex
Dim I As Integer

Dim Trace As Boolean

    Trace = False
    Omega = 2 * pi * F  ' get pulsation only once
    If Trace Then Debug.Print
    If Trace Then Debug.Print "Freq:", F
    
'''     Get source value
    Source = NetToComplex(TTrimAll(Form1.PlotZsrc.Text), Omega).Net
    If Trace Then Debug.Print "Source:", Source.re, Source.im

'''     Get reflected load
    Load.re = ZRatio
    Load.im = 0
    Load = CpxMul(Load, NetToComplex(TTrimAll(Form1.PlotZld.Text), Omega).Net)     'Reflected load
    If Trace Then Debug.Print "Load:", TTrimAll(Form1.PlotZld.Text), Load.re, Load.im
    
    
'''     Compute B (induction)
    V = Val(Form1.PlotVsource.Text)
    b = V * 10 ^ 4 / (4.44 * TbTurns(0) * FeSect * F)    'Induction at this voltage/frequency
    bac = b
    If Form1.ChkSE.Value Then b = b + Tmax / 2  'reserve tmax/2 for dc
    
'''     Using B and table, get "gapped" Mu
    Mu = Extrapol(b)
'    Debug.Print Tmax - bac,
    Gap = Val(Form1.TxGap.Text / 10) 'Displayed in mm, convert in cm
    Mu = Mu / (1 + (Mu * (Gap / FeLen)))    'Gapped Mu
    
'    Debug.Print Tmax - bac; "I0dc = "; Format(TbTurns(0) / Mu / FeLen, "####.####")
    ' Compute and check allowable Ip0

'''     Compute DC induction

'    If Form1.ChkSE.Value Then
'        b = b + Mu * Format(Form1.TxIp0.Text) * TbTurns(0) / FeLen / 10000       'Add DC flux
'        Mu = Extrapol(b)
'        Mu = Mu / (1 + (Mu * (Gap / FeLen)))
'    End If
    
    GetValVsF.BV = b
    GetValVsF.MuV = Mu

'''     Now, we can compute primary L value (LpV)
    GetValVsF.LpV = Mu * TbTurns(0) ^ 2 * FeSect * 10 ^ -8 / FeLen
    Lpri = Format(GetValVsF.LpV) & "H"

'''     The leakage inductance (already computed along with total shunt capacitance)
    LLeak = Format(LL / 1000 & "mH")
    CShunt = Format(CS * 1000 & "pF")
    Leak = NetToComplex(LLeak & "|" & CShunt & "| 5MR", Omega).Net
'**'    Leak = NetToComplex(LLeak, Omega).Net
    If Trace Then Debug.Print "Leak:", Leak.re, Leak.im

'''     The total wire resistance
    Rtot = Format(TbR(0) + TbR(1) * Sqr(ZRatio) & "R")
    
'''     Build the net from Output to Input
    Load = CpxPar(Load, LtoCpx(GetValVsF.LpV, F))   'Primary inductance in parallel with reflected load
'**'    Load = CpxPar(Load, NetToComplex(CShunt, Omega).Net) 'Shunt cap too
    Neta = CpxAdd(Load, Leak)  'Leak in serie
'''     The lump capacitance and inductance of each section having at least 2 layers (already computed)
    For I = 0 To UBound(Windings)
        If Windings(I).Layers > 1 Then
'''     One serie LC for each winding in parallel
            CLump = Format(Windings(I).Cap & "pF")
            LLump = Format(Windings(I).L & "mH")
'            Neta = CpxPar(Neta, NetToComplex(CLump & "+" & LLump & "+ 10KR", Omega).Net)
        End If
    Next
    Netb = CpxAdd(Neta, NetToComplex(Rtot, Omega).Net)   'Includes wires resistance
    Netb = CpxAdd(Netb, NetToComplex(CLump & "|" & LLump, Omega).Net)
    '''
'''    Netb = CpxPar(Netb, NetToComplex("10pF+70KR", Omega).Net)
    '''
    NetIn = CpxAdd(Netb, Source)
    Temp = CpxAdd(NetIn, Load)
    Temp = CpxDiv(Temp, Load)
'    Temp = CpxRatio(Load, NetIn)
    GetValVsF.BpV = -20 * Log10(Modulo(Temp))  'Level accross Load
    GetValVsF.PhV = 2 * Argument(Temp) 'Phase shift between source and load
    GetValVsF.ZpV = Modulo(Netb)
    
End Function

Private Sub XorPlot(Target As PictureBox, Vlist() As Single, K As Single, Color As Long)
'Draw or erase a curve from values in Vlist applying scale K
Dim I As Integer

    Target.ForeColor = Color
    Target.DrawMode = 13    'Xor Pen
    Target.CurrentY = K * Vlist(0)  'Begin of first segment
    Target.CurrentX = LogFlist(0)
    For I = 1 To PtoPlot    'and all others
        Target.Line -(LogFlist(I), K * Vlist(I))
    Next
    Target.DrawMode = 7
End Sub

Private Sub PlotToFile(b() As Single, Mu() As Single, Zp() As Single, Lp() As Single, PH() As Single, Bp() As Single)
' Create a file usable with a spreadsheet program.
Dim FName As String
Dim F As Integer
Dim I As Integer
Dim S As String

    If Len(Fichier) = 0 Then Exit Sub   'Not named yet
    FName = Substring(Fichier, 1, ".") & ".csv" 'Set extention to ".csv"
    F = IsOpen(FName, "W", 10)
    If F Then
        ' Indicate conditions
        Print #F, "Conditions :,Level =" & Form1.PlotVsource.Text & ",ZSource =" & Form1.PlotZsrc.Text & ",ZLoad =" & Form1.PlotZld.Text
        Print #F,
        Print #F, "Freq,Induction,Perm.,ZPri,LPri,Ph,Att(dB)"
        For I = 0 To PtoPlot  ' Set the list in 7 columns: Freq, B, Mu, Zp, Lp, Ph, Bp
            S = str(Flist(I)) & "," & str(b(I)) & "," & str(Mu(I)) & "," & str(Zp(I)) & "," & str(Lp(I)) & "," & str(PH(I) - 90) & "," & str(Bp(I))
            Print #F, S
        Next
        Print #F,
        XClose F, 10
    End If
End Sub

Private Sub ScalePlot(Vlist() As Single, K As Single)
Dim I As Integer

    For I = 0 To PtoPlot
        Vlist(I) = Vlist(I) * K
    Next

End Sub

Private Function ToCpx(T As String, W As Single) As Complex
' Returns the complex representation for the dipole in "T" for the specified w (pulsation)
' Where T can be:
' R10 : a 10 ohms resistor
' R1M : an 1 MegOhm resistor
' L1 : a 1 H inductor
' C2n : a 2 nanofarad capacitor
' and so on ....

Dim LS As Integer
Dim U As Single
Dim S As String
    
    S = T   'Don't modify T !
    Select Case Right(S, 1) 'Optionally, the last character may be the type:
    'F = Farad (implies a capacitance)
    'H = Henry (implies an inductance)
    'R = Resistor
    Case "F", "f"
        S = "C" & Left(S, Len(S) - 1) 'Remove the type char
    Case "H", "h"
        S = "L" & Left(S, Len(S) - 1)
    Case "R", "r"
        S = "R" & Left(S, Len(S) - 1)
    End Select
    
    Select Case Right(S, 1) 'the unity:
    'p or P = pico
    'n or N = nano
    ' = micro
    'm = milli
    'k or K = Kilo
    'M = Mega
    Case "p", "P"
        U = 0.000000000001 '1 / 1000000000000# '0.00000000001
        S = Left(S, Len(S) - 1) 'remove the unity char
    Case "n", "N"
        U = 0.000000001
        S = Left(S, Len(S) - 1)
    Case ""
        U = 0.000001
        S = Left(S, Len(S) - 1)
    Case "m"
        U = 0.001
        S = Left(S, Len(S) - 1)
    Case "K", "k"
        U = 1000
        S = Left(S, Len(S) - 1)
    Case "M"
        U = 1000000
        S = Left(S, Len(S) - 1)
    Case Else
        U = 1
    End Select
    
    LS = Len(S)
    Select Case UCase(Left(S, 1))
    Case "R"
        ToCpx.re = U * Val(Right(S, LS - 1))
        ToCpx.im = 0
    Case "L"
        ToCpx.re = 0
        ToCpx.im = U * Val(Right(S, LS - 1)) * W
    Case "C"
        ToCpx.re = 0
        ToCpx.im = 1 / -(U * Val(Right(S, LS - 1)) * W)
    End Select
End Function
Private Function InP(Source As String, Search As String, start As Integer) As Integer
' Search backward for "Search" in "String" returning where it was found.
Dim N As Integer
Dim Bopen As Integer
Dim Bclose As Integer
Dim Sign As String

    N = start
    If N > Len(Source) Then N = Len(Source) - Len(Search) + 1
    Bopen = 0
    Bclose = 0
    While N
        Sign = Mid(Source, N, Len(Search))
        If (Sign = Search) And (Bopen = Bclose) Then
            InP = N
            Exit Function
        End If
        If Left(Sign, 1) = "(" Then Bopen = Bopen + 1
        If Left(Sign, 1) = ")" Then Bopen = Bopen - 1
        N = N - 1
    Wend
    InP = 0
End Function


Private Function NetToComplex(S As String, W As Single) As XNet
' Returns the complex number represented by the expression in S at the pulsation W

Dim I As Integer
Dim J As Integer
Dim P As Integer
Dim SRight As String
Dim SLeft As String
Dim Op As String
Dim Ops As String
Dim Dummy As Double

    On Error GoTo NETERROR
    Ops = "+|"  ' 'Operators list in ascending priority order
    For J = 1 To Len(Ops)
        Op = Mid(Ops, J, 1)
        P = InP(UCase(S), Op, 255)
        While P > 0     'Found an Op
            SLeft = Left(S, P - 1)
            SRight = Right(S, Len(S) - P)
            Select Case Op
            Case "+"        'Means serialized terms
                NetToComplex.Net = CpxAdd(GetNet(NetToComplex(SRight, W)), GetNet(NetToComplex(SLeft, W)))
                Exit Function
            Case "|"      'Means paralleled terms
                NetToComplex.Net = CpxPar(GetNet(NetToComplex(SRight, W)), GetNet(NetToComplex(SLeft, W)))
                Exit Function
            End Select
        Wend
    Next
'Check if expression starts and ends with parenthesis
    If Left(S, 1) = "(" And Right(S, 1) = ")" Then
        S = Mid(S, 2, Len(S) - 2)   'Remove them
        NetToComplex = NetToComplex(S, W) 'And evaluate it
        Exit Function
    End If
' No more operator, evaluates and returns the operand
    NetToComplex.Net = ToCpx(S, W)
    Exit Function
NETERROR:
    SetFatal "NETERROR"
End Function


Private Function GetNet(N As XNet) As Complex

    GetNet = N.Net
End Function
